@@ -19,26 +19,29 @@ DEBUG_I2C = 0
 class IOBoard:
   "This class controls the N2ADR IO Board for the HermesLite 2"
   def __init__(self, hardware):
+    self.DEBUG = 0
     self.hardware = hardware
     self.have_IO_Board = None
     self.have_board_counter = 3
     self.tx_timer = 0
     self.current_tx_freq = 0
+    self.old_receive = None
   def HeartBeat(self):	# Called at 10 Hz for housekeeping tasks
-    return
     if not QS.get_params('rx_udp_started'):
       return
     if self.have_IO_Board is None:
       resp = self.hardware.ReadI2C(0x7d, 0x41, 0)	# Check for the N2ADR HL2 IO Board
       if resp and resp[1] == 0xF1:
         self.have_IO_Board = True
-        if True or DEBUG_I2C: print ('Have IO_Board')
+        if self.DEBUG or DEBUG_I2C:
+          print ('Have IO_Board')
         self.hardware.ImmediateChange('hermes_iob_rxin')
       else:
         self.have_board_counter -= 1
         if self.have_board_counter == 0:
           self.have_IO_Board = False
-          if True or DEBUG_I2C: print ('No IO board')
+          if self.DEBUG or DEBUG_I2C:
+            print ('No IO board')
     if not self.have_IO_Board:
       return
     if self.hardware.tx_frequency != self.current_tx_freq and time.time() - self.tx_timer > 0.50:
@@ -50,12 +53,15 @@ class IOBoard:
       self.hardware.WriteI2C(0x7d, 0x1D, 2, (tx >> 16) & 0xFF)
       self.hardware.WriteI2C(0x7d, 0x1D, 3, (tx >>  8) & 0xFF)
       self.hardware.WriteI2C(0x7d, 0x1D, 13, tx        & 0xFF)	# LSB
-      #self.Receive()
+    if self.DEBUG:
+      ret = self.Receive()
+      if ret and ret != self.old_receive:
+        self.old_receive = ret
+        print ('IO Board: 0x%X 0x%X 0x%X 0x%X 0x%X' % tuple(ret))
   def Receive(self, register=0):	# Get the response from the IO board
     if not self.have_IO_Board:
-      return
+      return None
     ret = self.hardware.ReadI2C(0x7d, 0x1D, register)
-    #print ('IO Board: 0x%X 0x%X 0x%X 0x%X 0x%X' % tuple(ret))
     return ret
   def FanLevel(self, level):
     if not self.have_IO_Board:
@@ -959,7 +965,7 @@ class Hardware(BaseHardware):
         if resp[0] != 0:
           #print ("0x%X 0x%X 0x%X 0x%X 0x%X" % tuple(resp))
           if bus & 0x3F == (resp[0] >> 1) & 0x3F:
-            if DEBUG_I2C or DEBUG:
+            if DEBUG_I2C > 1 or DEBUG:
               print ("Write I2C bus OK 0x%X, i2caddr 0x%X, control 0x%X, value 0x%X" % (bus, i2caddr, control, value))
           else:
             if DEBUG_I2C or DEBUG:
@@ -969,7 +975,7 @@ class Hardware(BaseHardware):
         if DEBUG_I2C or DEBUG:
           print ("Write I2C bus TIMEOUT 0x%X, i2caddr 0x%X, control 0x%X, value 0x%X" % (bus, i2caddr, control, value))
     else:
-      if True or DEBUG: print ("Write I2C bus 0x%X, i2caddr 0x%X, control 0x%X, value 0x%X" % (bus, i2caddr, control, value))
+      if DEBUG_I2C > 1 or DEBUG: print ("Write I2C bus 0x%X, i2caddr 0x%X, control 0x%X, value 0x%X" % (bus, i2caddr, control, value))
   def ReadI2C(self, bus, i2caddr, control):
     # bus is 0x7c or 0x7d
     self.WriteQueue(0)		# Finish any writes
