@@ -984,7 +984,8 @@ void quisk_hermes_tx_send(int tx_socket, int * tx_records)
 	sendbuf[522] = 0x7F;
 
 	// Changes for HermesLite v2 thanks to Steve, KF7O
-	if ((quisk_hermeslite_writepointer > 0) && (quisk_hermeslite_writeattempts++ % 8 == 0)) {
+	// Add an initial delay in sending ACK request so two successive requests are spaced.
+	if ((quisk_hermeslite_writepointer > 0) && (quisk_hermeslite_writeattempts % 8 == 7)) {
 
 		// Only send periodic hermeslite writes in second part of frame
 		hlwp = 5*(quisk_hermeslite_writepointer-1);
@@ -1017,13 +1018,12 @@ void quisk_hermes_tx_send(int tx_socket, int * tx_records)
                 }
 		if (++C0_index > 16)
 			C0_index = 0;
-	
-		if (quisk_hermeslite_writepointer > 0) quisk_hermeslite_writeattempts++;
 	}
+	if (quisk_hermeslite_writepointer > 0) quisk_hermeslite_writeattempts++;
 
 	// Abort after 53/8 ~= 5 retries
 	if ((quisk_hermeslite_writepointer > 0) && (quisk_hermeslite_writeattempts > 53)) {
-		QuiskPrintf("ERROR: Maximum Hermes-Lite write attempts\n");
+		QuiskPrintf("ERROR: Maximum Hermes-Lite write attempts without ACK\n");
 		// Cancel entire write sequence
 		quisk_hermeslite_writepointer = 0;
 		quisk_hermeslite_writeattempts = 0;
@@ -1251,7 +1251,7 @@ int quisk_process_microphone(int mic_sample_rate, complex double * cSamples, int
 	}
 	if (key_down != key_was_down) {	
 		key_down_counter = quisk_start_ssb_delay * 48;	// Key was pressed. Zero the first few samples to clear the buffers.
-		init_alc(&tx_alc, 0);		// init ALC for Tx and RECORD_MIC
+		init_alc(&tx_alc, 0);		// init ALC for Tx and TMP_RECORD_MIC
 		key_was_down = key_down;
 	}
 	if (quisk_play_state != last_play_state) {
@@ -1269,21 +1269,21 @@ int quisk_process_microphone(int mic_sample_rate, complex double * cSamples, int
 		else switch (rxMode) {
 		case LSB:		// LSB
 		case USB:		// USB
-			if (quisk_record_state == PLAYBACK)
+			if (quisk_record_state == TMP_PLAY_SPKR_MIC)
 				count = tx_filter_digital(cSamples, count);	// filter samples, minimal processing
 			else
 				count = tx_filter(cSamples, count);		// filter samples
 			process_alc(cSamples, count, &tx_alc, rxMode);
 			break;
 		case AM:		// AM
-			if (quisk_record_state != PLAYBACK)		// no audio processing for recorded sound
+			if (quisk_record_state != TMP_PLAY_SPKR_MIC)		// no audio processing for recorded sound
 				count = tx_filter(cSamples, count);
 			for (i = 0; i < count; i++)	// transmit (0.5 + ampl/2, 0)
 				cSamples[i] = (creal(cSamples[i]) + CLIP16) * 0.5;
 			process_alc(cSamples, count, &tx_alc, rxMode);
 			break;
 		case FM:		// FM
-			if (quisk_record_state != PLAYBACK)		// no audio processing for recorded sound
+			if (quisk_record_state != TMP_PLAY_SPKR_MIC)		// no audio processing for recorded sound
 				count = tx_filter(cSamples, count);
 			if (quisk_ctcss_freq > 9) {
 				ctcss_delta = 2.0 * M_PI / MIC_OUT_RATE * quisk_ctcss_freq;
@@ -1394,7 +1394,7 @@ int quisk_process_microphone(int mic_sample_rate, complex double * cSamples, int
 	else if (quisk_use_rx_udp && key_down) {	// Send mic samples to UDP when key is down
 		transmit_udp(cSamples, count);
 	}
-	if (quisk_record_state == RECORD_MIC) {
+	if (quisk_record_state == TMP_RECORD_MIC) {
 		switch (rxMode) {
 		case LSB:		// LSB
 		case USB:		// USB
