@@ -2,16 +2,38 @@
 # It provides USB control of SoftRock hardware.
 
 from __future__ import print_function
+from __future__ import absolute_import
+from __future__ import division
 
-import struct, threading, time, traceback, math
+import sys, struct, time, traceback, math
 from quisk_hardware_model import Hardware as BaseHardware
 import _quisk as QS
 
 # All USB access is through control transfers using pyusb.
-#   byte_array      = dev.ctrl_transfer (IN,  bmRequest, wValue, wIndex, length, timout)
-#   len(string_msg) = dev.ctrl_transfer (OUT, bmRequest, wValue, wIndex, string_msg, timout)
+#   byte_array      = dev.ctrl_transfer (IN,  bmRequest, wValue, wIndex, length, timeout)
+#   len(string_msg) = dev.ctrl_transfer (OUT, bmRequest, wValue, wIndex, string_msg, timeout)
 
-import usb.core, usb.util
+try:
+  import usb
+  import usb.core, usb.util
+except:
+  if sys.platform == 'win32':
+    dlg = wx.MessageDialog(None, "The Python pyusb module is required but not installed. Do you want me to install it?",
+      "Install Python pyusb", style = wx.YES|wx.NO)
+    if dlg.ShowModal() == wx.ID_YES:
+      import subprocess
+      subprocess.call([sys.executable, "-m", "pip", "install", "pyusb"])
+      try:
+        import usb
+        import usb.core, usb.util
+      except:
+        dlg = wx.MessageDialog(None, "Installation of Python pyusb failed. Please install it by hand.",
+           "Installation failed", style=wx.OK)
+        dlg.ShowModal()
+  else:
+    dlg = wx.MessageDialog(None, "The Python pyusb module is required but not installed. Please install package python-usb.",
+      "Install Python pyusb", style = wx.OK)
+    dlg.ShowModal()
 
 DEBUG = 0
 
@@ -36,40 +58,62 @@ class Hardware(BaseHardware):
     BaseHardware.__init__(self, app, conf)
     self.usb_dev = None
     self.vfo = None
+    self.mode = None
+    self.band = None
+    self.repeater_freq = None		# original repeater output frequency
+    try:
+      self.repeater_delay = conf.repeater_delay		# delay for changing repeater frequency in seconds
+    except:
+      self.repeater_delay = 0.25
+    self.repeater_time0 = 0			# time of repeater change in frequency
     self.ptt_button = 0
-    self.is_cw = False
-    self.key_thread = None
     self.si570_i2c_address = conf.si570_i2c_address
   def open(self):			# Called once to open the Hardware
+    global usb
     # find our device
     usb_dev = usb.core.find(idVendor=self.conf.usb_vendor_id, idProduct=self.conf.usb_product_id)
     if usb_dev is None:
       text = 'USB device not found VendorID 0x%X ProductID 0x%X' % (
           self.conf.usb_vendor_id, self.conf.usb_product_id)
-      self.application.sound_error = 1
     else:
       try:		# This exception occurs for the Peabody SDR.  Thanks to ON7VQ for figuring out the problem,
         usb_dev.set_configuration()        # and to David, AE9RB, for the fix.
       except:
-        if DEBUG: traceback.print_exc()
+        pass #if DEBUG: traceback.print_exc()
       try:
         ret = usb_dev.ctrl_transfer(IN, 0x00, 0x0E00, 0, 2)
       except:
         if DEBUG: traceback.print_exc()
         text = "No permission to access the SoftRock USB interface"
-        self.application.sound_error = 1
-      else:
+        usb_dev = None
+        try:	# Second try. Thanks to Ben Cahill.
+          if DEBUG: print("Second try to open USB with libusb0 backend.")
+          import usb.backend.libusb0
+          backend = usb.backend.libusb0.get_backend()
+          usb_dev = usb.core.find(idVendor=self.conf.usb_vendor_id, idProduct=self.conf.usb_product_id, backend=backend)
+          try:
+            usb_dev.set_configuration()
+          except:
+            if DEBUG: traceback.print_exc()
+          try:
+            ret = usb_dev.ctrl_transfer(IN, 0x00, 0x0E00, 0, 2)
+          except:
+            if DEBUG: traceback.print_exc()
+            usb_dev = None
+        except:
+          usb_dev = None
+    if usb_dev is not None:
         self.usb_dev = usb_dev		# success
         if len(ret) == 2:
           ver = "%d.%d" % (ret[1], ret[0])
         else:
           ver = 'unknown'
-        text = 'Capture from SoftRock USB on %s, Firmware %s' % (self.conf.name_of_sound_capt , ver)
-        if self.conf.name_of_mic_play and self.conf.key_poll_msec:
-          self.key_thread = KeyThread(usb_dev, self.conf.key_poll_msec / 1000.0, self.conf.key_hang_time)
-          self.key_thread.start()
-    self.application.bottom_widgets.info_text.SetLabel(text)
-    if DEBUG:
+        sound = self.conf.name_of_sound_capt
+        if len(sound) > 50:
+          sound = sound[0:30] + '|||' + sound[-17:]
+        text = 'Capture from SoftRock USB on %s, Firmware %s' % (sound, ver)
+    #self.application.bottom_widgets.info_text.SetLabel(text)
+    if DEBUG and usb_dev:
       print ('Startup freq', self.GetStartupFreq())
       print ('Run freq', self.GetFreq())
       print ('Address 0x%X' % usb_dev.ctrl_transfer(IN, 0x41, 0, 0, 1)[0])
@@ -78,70 +122,86 @@ class Hardware(BaseHardware):
       print ('Smooth tune', sm)
     return text
   def close(self):			# Called once to close the Hardware
-    if self.key_thread:
-      self.key_thread.stop()
-      self.key_thread = None
+    pass
   def ChangeFrequency(self, tune, vfo, source='', band='', event=None):
     if self.usb_dev and self.vfo != vfo:
       if self.conf.si570_direct_control:
-        if self.SetFreqByDirect(vfo):
+        if self.SetFreqByDirect(vfo - self.transverter_offset):
           self.vfo = vfo
-      elif self.SetFreqByValue(vfo):
+      elif self.SetFreqByValue(vfo - self.transverter_offset):
          self.vfo = vfo
       if DEBUG:
         print ('Change to', vfo)
         print ('Run freq', self.GetFreq())
     return tune, vfo
+  def ChangeBand(self, band):
+    # band is a string: "60", "40", "WWV", etc.
+    BaseHardware.ChangeBand(self, band)
+    self.band = band
+    self.SetTxLevel()
+  def ChangeMode(self, mode):
+    # mode is a string: "USB", "AM", etc.
+    BaseHardware.ChangeMode(self, mode)
+    self.mode = mode
+    QS.set_cwkey(0)
+    self.SetTxLevel()
+  def SetTxLevel(self):
+    tx_level = self.conf.tx_level.get(self.band, 70)
+    if self.mode[0:3] in ('DGT', 'FDV'):			# Digital modes; change power by a percentage
+      reduc = self.application.digital_tx_level
+    else:
+      reduc = self.application.tx_level
+    tx_level = int(tx_level * reduc / 100.0 + 0.5)  
+    if tx_level < 0:
+      tx_level = 0
+    elif tx_level > 100:
+      tx_level = 100
+    QS.set_mic_out_volume(tx_level)
+    if DEBUG: print("Change tx_level to", tx_level)
   def ReturnFrequency(self):
     # Return the current tuning and VFO frequency.  If neither have changed,
     # you can return (None, None).  This is called at about 10 Hz by the main.
     # return (tune, vfo)	# return changed frequencies
     return None, None		# frequencies have not changed
-  def ChangeMode(self, mode):		# Change the tx/rx mode
-    # mode is a string: "USB", "AM", etc.
-    if mode in ('CWU', 'CWL'):
-      self.is_cw = True
-    else:
-      self.is_cw = False
-    if self.key_thread:
-      self.key_thread.IsCW(self.is_cw)
-    elif hasattr(self, 'OnButtonPTT'):
-      self.OnButtonPTT()
-  def ChangeBand(self, band):
-    # band is a string: "60", "40", "WWV", etc.
-    pass
+  def RepeaterOffset(self, offset=None):	# Change frequency for repeater offset during Tx
+    if offset is None:		# Return True if frequency change is complete
+      if time.time() > self.repeater_time0 + self.repeater_delay:
+        return True
+    elif offset == 0:			# Change back to the original frequency
+      if self.repeater_freq is not None:
+        self.repeater_time0 = time.time()
+        self.ChangeFrequency(self.repeater_freq, self.repeater_freq, 'repeater')
+        self.repeater_freq = None
+    else:			# Shift to repeater input frequency
+      self.repeater_time0 = time.time()
+      self.repeater_freq = self.vfo
+      vfo = self.vfo + int(offset * 1000)	# Convert kHz to Hz
+      self.ChangeFrequency(vfo, vfo, 'repeater')
+    return False
   def OnSpot(self, level):
-    if self.key_thread:
-      self.key_thread.OnSpot(level)
+    pass
   def HeartBeat(self):	# Called at about 10 Hz by the main
     pass
-  def OnButtonPTT(self, event=None):
-    if event:
-      if event.GetEventObject().GetValue():
-        self.ptt_button = 1
-      else:
-        self.ptt_button = 0
-    if self.key_thread:
-      self.key_thread.OnPTT(self.ptt_button)
-    elif self.usb_dev:
-      if self.is_cw:
-        QS.set_key_down(0)
-        QS.set_transmit_mode(self.ptt_button)
-      else:
-        QS.set_key_down(self.ptt_button)
+  def OnChangeRxTx(self, is_tx):	# Called by Quisk when changing between Rx and Tx. "is_tx" is 0 or 1
+    if not self.usb_dev:
+      return
+    try:
+      self.usb_dev.ctrl_transfer(IN, 0x50, is_tx, 0, 3)
+    except usb.core.USBError:
+      if DEBUG: traceback.print_exc()
       try:
-        self.usb_dev.ctrl_transfer(IN, 0x50, self.ptt_button, 0, 3)
+        self.usb_dev.ctrl_transfer(IN, 0x50, is_tx, 0, 3)
       except usb.core.USBError:
         if DEBUG: traceback.print_exc()
-        try:
-          self.usb_dev.ctrl_transfer(IN, 0x50, self.ptt_button, 0, 3)
-        except usb.core.USBError:
-          if DEBUG: traceback.print_exc()
+      else:
+        if DEBUG: print("OnChangeRxTx", is_tx)
+    else:
+      if DEBUG: print("OnChangeRxTx", is_tx)
   def GetStartupFreq(self):	# return the startup frequency / 4
     if not self.usb_dev:
       return 0
     ret = self.usb_dev.ctrl_transfer(IN, 0x3C, 0, 0, 4)
-    s = ret.tostring()
+    s = ret.tobytes()
     freq = UBYTE4.unpack(s)[0]
     freq = int(freq * 1.0e6 / 2097152.0 / 4.0 + 0.5)
     return freq
@@ -149,7 +209,7 @@ class Hardware(BaseHardware):
     if not self.usb_dev:
       return 0
     ret = self.usb_dev.ctrl_transfer(IN, 0x3A, 0, 0, 4)
-    s = ret.tostring()
+    s = ret.tobytes()
     freq = UBYTE4.unpack(s)[0]
     freq = int(freq * 1.0e6 / 2097152.0 / 4.0 + 0.5)
     return freq
@@ -179,7 +239,7 @@ class Hardware(BaseHardware):
       if n1 < 1:
         n1 = 1
       else:
-        n1 = ((n1 + 1) / 2) * 2
+        n1 = ((n1 + 1) // 2) * 2
       dco = (freq * 1.0) * hsdiv * n1
       # Since we're starting with max hsdiv, this can only happen if
       # freq was larger than we can handle
@@ -210,64 +270,19 @@ class Hardware(BaseHardware):
                                    ((rfreq_int & 0xf) << 28) + rfreq_frac)
     self.usb_dev.ctrl_transfer(OUT, 0x30, self.si570_i2c_address + 0x700, 0, s)
     return True		# Success
-
-class KeyThread(threading.Thread):
-  """Create a thread to monitor the key state."""
-  def __init__(self, dev, poll_secs, key_hang_time):
-    self.usb_dev = dev
-    self.poll_secs = poll_secs
-    self.key_hang_time = key_hang_time
-    self.ptt_button = 0
-    self.spot_level = 0
-    self.currently_in_tx = 0
-    self.is_cw = False
-    self.key_timer = 0
-    self.is_transmit = 0
-    threading.Thread.__init__(self)
-    self.doQuit = threading.Event()
-    self.doQuit.clear()
-  def run(self):
-    while not self.doQuit.isSet():
-      if self.is_cw:
-        try:		# Test key up/down state
-          ret = self.usb_dev.ctrl_transfer(IN, 0x51, 0, 0, 1)
-        except usb.core.USBError:
-          if DEBUG: traceback.print_exc()
-        else:
-          # bit 0x20 is the tip, bit 0x02 is the ring (ring not used)
-          if self.spot_level or ret[0] & 0x20 == 0:		# Tip: key is down
-            QS.set_key_down(1)
-            self.is_transmit = 1
-            self.key_timer = time.time()
-          else:			# key is up
-            QS.set_key_down(0)
-            if self.is_transmit and time.time() - self.key_timer > self.key_hang_time:
-              self.is_transmit = 0
-          if self.is_transmit != self.currently_in_tx:
-            try:
-              self.usb_dev.ctrl_transfer(IN, 0x50, self.is_transmit, 0, 3)
-            except usb.core.USBError:
-              if DEBUG: traceback.print_exc()
-            else:
-              self.currently_in_tx = self.is_transmit	# success
-              QS.set_transmit_mode(self.is_transmit)
-              if DEBUG: print ("Change currently_in_tx", self.currently_in_tx)
-      elif self.ptt_button != self.currently_in_tx:
-        QS.set_key_down(self.ptt_button)
-        try:
-          self.usb_dev.ctrl_transfer(IN, 0x50, self.ptt_button, 0, 3)
-        except usb.core.USBError:
-          if DEBUG: traceback.print_exc()
-        else:
-          self.currently_in_tx = self.ptt_button	# success
-          if DEBUG: print ("Change currently_in_tx", self.currently_in_tx)
-      time.sleep(self.poll_secs)
-  def stop(self):
-    """Set a flag to indicate that the thread should end."""
-    self.doQuit.set()
-  def OnPTT(self, ptt):
-    self.ptt_button = ptt
-  def OnSpot(self, level):
-    self.spot_level = level
-  def IsCW(self, is_cw):
-    self.is_cw = is_cw
+  def PollCwKey(self):  # Called frequently by Quisk to check the CW key status
+    if not self.usb_dev:
+      return
+    if self.mode not in ('CWU', 'CWL'):
+      return
+    try:		# Test key up/down state
+      ret = self.usb_dev.ctrl_transfer(IN, 0x51, 0, 0, 1)
+    except:
+      QS.set_cwkey(0)
+      if DEBUG: traceback.print_exc()
+    else:
+      # bit 0x20 is the tip, bit 0x02 is the ring (ring not used)
+      if ret[0] & 0x20 == 0:		# Tip: key is down
+        QS.set_cwkey(1)
+      else:			# key is up
+        QS.set_cwkey(0)
